local addon, ns = ...

-- got Class??
local pClass = select(2,UnitClass("player"))

KokoSpellAuras = LibStub("AceAddon-3.0"):NewAddon("KokoSpellAuras", "AceConsole-3.0", "AceEvent-3.0")
local SpellActivationOverlay_OnEvent = SpellActivationOverlay_OnEvent

-- Shared Media
local media = LibStub("LibSharedMedia-3.0")

-- saved variable references
-- ns.DB()        main settings
-- ns.AuraDB()    blizzard overlays
-- Reset          Pass value, eg: ns.DB(true)
-----

local kSA_Loaded = false

local NewAuraQueue = {}

-- holder frames
local overlayFrame

local function loadSavedMedia()
   -- loads collected auras into table for faster position lookup
   -- also registers collected auras into shared media
   -- We only do this once per UI load
   local DB = { ns.AuraDB(), ns.CustomDB() }
   
   for _,v in pairs(DB) do
	   for class, spec in pairs(v) do
		  for specID, auraL in pairs(spec) do
			 for auraID, val in pairs(auraL) do
				ns.RegisterTexture(val[1], select(1, GetSpellInfo(auraID)))
				ns.texPosInsert(ns.BlizTexName(val[1]), val[2])
			 end
		  end
	   end
   end

end

function KokoSpellAuras:ToggleOverlayFrame(val)
   if val then
      overlayFrame:Show()
   else
      overlayFrame:Hide()
   end
   ns.showoverlay = val
end

function KokoSpellAuras:Init()
   --init database
   ns.DB()
   ns.AuraDB()
   ns.CleanUp()
   
   -- load media
   loadSavedMedia()
   
   if ns.DB().enable then
      self:OnEnable()
   end
   
end

function KokoSpellAuras:OnInitialize()
   -- Called when the addon is loaded
   LibStub("AceConfig-3.0"):RegisterOptionsTable("KokoSpellAuras", ns.options)
   self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("KokoSpellAuras", "Koko Spell Auras")

   -- init settings
   self:Init()
   ns.buildMenu()   
   -- Chat Commands
   self:RegisterChatCommand("ksa", "ChatCommand")
    -- LDB launcher
   LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("KokoSpellAuras", {
      type = "launcher",
      icon = media:Fetch('spelloverlay', 'Maelstrom_Weapon'),
      text = "Koko Spell Auras",
      OnClick = function(clickedframe, button)
         if IsShiftKeyDown() then
            KokoSpellAuras:ToggleOverlayFrame(not ns.showoverlay)
         elseif button == 'LeftButton' then
            LibStub("AceConfigDialog-3.0"):Open("KokoSpellAuras")
         elseif button == 'RightButton' then
            LibStub("AceConfigDialog-3.0"):Close("KokoSpellAuras")
         end
      end,
      OnLeave = function() return GameTooltip:Hide() end,
      OnEnter = function(self) 
         local frame = self
         local showBelow = select( 2, self:GetCenter() ) > UIParent:GetHeight() / 2
         GameTooltip:SetOwner( self, "ANCHOR_NONE" )
         GameTooltip:SetPoint( showBelow and "TOP" or "BOTTOM", self, showBelow and "BOTTOM" or "TOP" )
         GameTooltip:AddLine( "Left/right click to show/hide options.")
         GameTooltip:AddLine( "Shift click to toggle overlay resizing.")
         return GameTooltip:Show()
      end,
   })
   
   self:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED')
end

function KokoSpellAuras:ChatCommand(input)
   LibStub("AceConfigDialog-3.0"):Open("KokoSpellAuras")
end

function KokoSpellAuras:PLAYER_SPECIALIZATION_CHANGED()
   LibStub("AceConfigRegistry-3.0"):NotifyChange("KokoSpellAuras")
end

function KokoSpellAuras:ProcessAuraOverlay(pClass, specID, spellID, custom)
   -- Sound, we play this first.
   PlaySoundFile(media:Fetch('sound', ns.GetOptions['sound'](pClass, specID, spellID)))
   -- Display
   local tex, loc, scale, r, g, b, hFlip, vFlip, height, width = ns.GetOverlayCfg(pClass, specID, spellID)
   scale = scale*ns.sizeScale -- default size scaling
   tex = media:Fetch('spelloverlay', tex)
   
   loc = string.upper(loc)
      
   if (ns.complexLocationTable[loc] ) then
      for location, info in pairs(ns.complexLocationTable[loc]) do
         local flipv = info.vFlip
         local fliph = info.hFlip
         if vFlip then
            flipv = not info.vFlip
         end
         if hFlip then
            fliph = not info.hFlip
         end
         SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, spellID, tex, location, scale, r, g, b, flipv, fliph);
         -- sizing
         SpellActivationOverlay_GetOverlay(SpellActivationOverlayFrame, spellID, location):SetSize(width * scale, height * scale)
      end
      
   else
      SpellActivationOverlay_ShowOverlay(SpellActivationOverlayFrame, spellID, tex, loc, scale, r, g, b, vFlip, hFlip);
      -- sizing
      SpellActivationOverlay_GetOverlay(SpellActivationOverlayFrame, spellID, loc):SetSize(width * scale, height * scale)
   end
end

function KokoSpellAuras:OnOverlayEvent(self, event, spellID, ...)
   if not ns.DB().enable then return end
   if event == "SPELL_ACTIVATION_OVERLAY_SHOW" then
       -- get class specID
      local specID = GetSpecializationInfo(GetSpecialization())
      
	  	local AuraDB
		if select(7,...) == true then
			AuraDB = ns.CustomDB()
		else
			AuraDB = ns.AuraDB()
		end
	  
      if not AuraDB[pClass][specID] then
         AuraDB[pClass][specID] = {}
      end
	  
      -- collect
      if ns.DB()['aCol'] then
         if not AuraDB[pClass][specID][spellID] then
            -- new aura, queue data for load on combat exit
            table.insert(NewAuraQueue, {specID, spellID, ...})
            KokoSpellAuras:RegisterEvent('PLAYER_REGEN_ENABLED')
         end
         AuraDB[pClass][specID][spellID] = {...}
      end

      -- exit if aura not yet exist or queued
      for index, val in pairs(NewAuraQueue) do
         if NewAuraQueue[index][2] == spellID then
            -- show collected blizzard overlay
            SpellActivationOverlay_ShowAllOverlays(self, spellID, ...)
            return
         end
      end
      
      if not AuraDB[pClass][specID][spellID] then return end
      
      -- process aura settings
      KokoSpellAuras:ProcessAuraOverlay(pClass, specID, spellID, select(7,...))
   elseif  event == "SPELL_ACTIVATION_OVERLAY_HIDE"  then
      if spellID then
         SpellActivationOverlay_HideOverlays(self, spellID);
      else
         SpellActivationOverlay_HideAllOverlays(self);
      end
   end

end

function KokoSpellAuras:OnEnable()
   if not kSA_Loaded then
      
      -- make overlay helper frame
      overlayFrame = CreateFrame('Frame', nil, UIParent)
      overlayFrame:SetAllPoints(SpellActivationOverlayFrame)
      overlayFrame:SetBackdrop({ bgFile = [[Interface\Tooltips\UI-Tooltip-Background]] })
      overlayFrame:SetBackdropColor(0.7, 0, 0.5, 0.4)
      
      -- x/y guide lines
      local guides = { 'x', 'y' }
      local offset = 50
      for i = 1, #guides do
         overlayFrame[guides[i]] = CreateFrame('Frame', nil, UIParent)
         overlayFrame[guides[i]]:SetParent(overlayFrame)
         overlayFrame[guides[i]]:SetBackdrop({ bgFile = [[Interface\Tooltips\UI-Tooltip-Background]] })
         overlayFrame[guides[i]]:SetBackdropColor(1, 1, 1, 1)
         overlayFrame[guides[i]]:SetSize(1,1)
         overlayFrame[guides[i]]:SetPoint('center', overlayFrame)
      end
      
      overlayFrame.x:SetPoint('left', overlayFrame, 'left', -offset, 0)
      overlayFrame.x:SetPoint('right', overlayFrame, 'right', offset, 0)
      overlayFrame.y:SetPoint('top', overlayFrame, 'top', 0, offset)
      overlayFrame.y:SetPoint('bottom', overlayFrame, 'bottom', 0, -offset)
      
      --mouse sizing controls
      overlayFrame:EnableMouse(true)
      
      overlayFrame:SetScript('OnEnter', function(self) 
         GameTooltip:SetOwner( self, "ANCHOR_NONE" )
         GameTooltip:SetPoint( 'bottom', self, 'top', 0, 128)
         GameTooltip:AddLine( "Resizing Controls" )
         GameTooltip:AddLine( "Height: Mousewheel up + down" )
         GameTooltip:AddLine( "Width: Shift + Mousewheel up + down" )
         GameTooltip:AddLine( "Alt: Increases size value" )
         return GameTooltip:Show()
      end)
      overlayFrame:SetScript('OnLeave', function() 
         return GameTooltip:Hide()
      end)
      
      overlayFrame:SetScript('OnMouseWheel', function(_, val) 
         local h = ns.DB()['OverlaySize']['height']
         local w = ns.DB()['OverlaySize']['width']
         
         if IsAltKeyDown() then
            val = val*10
         end
         
         if IsShiftKeyDown() then
            -- width
            val = w + val
            SpellActivationOverlayFrame:SetWidth(val)
            ns.DB()['OverlaySize']['width'] = val
         else
            -- height
            val = h + val
            SpellActivationOverlayFrame:SetHeight(val)
            ns.DB()['OverlaySize']['height'] = val
         end
      end)
      
      -- show/hide generate random textures
      local location = { 'TOP', 'SIDES' }
      local pos = { 'TOP' , 'Left + Right (Flipped)' }
      overlayFrame:SetScript('OnShow', function(self)
         if ns.DB().enable then
            -- generate random procs for top + sides
            for i = 1, #location do
               local rngTex = media:Fetch('spelloverlay', ns.getRandomTex(location[i]))
               SpellActivationOverlay_ShowAllOverlays(SpellActivationOverlayFrame, i, rngTex, pos[i], 1, 255, 255, 255)
            end
            -- bottom
            local rngTex = media:Fetch('spelloverlay', ns.getRandomTex('TOP'))
            SpellActivationOverlay_ShowAllOverlays(SpellActivationOverlayFrame, #location+1, rngTex, 'BOTTOM', 1, 255, 255, 255)
         end
      end)
      overlayFrame:SetScript('OnHide', function(self)
         for i = 1, #location+1 do
               SpellActivationOverlay_HideOverlays(SpellActivationOverlayFrame, i)
            end
         end)
      overlayFrame:Hide()

      kSA_Loaded = true
   end
   
   -- over-rides blizzard overlays, this avoids graphic "jumping" between locations for non default alerts
   SpellActivationOverlayFrame:SetScript("OnEvent", function(...) KokoSpellAuras:OnOverlayEvent(...) end)
   
   -- set overlay x/y
   SpellActivationOverlayFrame:ClearAllPoints()
   SpellActivationOverlayFrame:SetPoint("center", UIParent)
   SpellActivationOverlayFrame:SetSize(ns.DB()['OverlaySize']['width'], ns.DB()['OverlaySize']['height'])
   SpellActivationOverlayFrame:SetFrameStrata(ns.DB()['OverlaySize']['strata'])
   -- hide sizing overlayFrame on combat
   self:RegisterEvent('PLAYER_REGEN_DISABLED')
end

function KokoSpellAuras:OnDisable()
   -- return overlay frame to original size, location and strata
   SpellActivationOverlayFrame:ClearAllPoints()
   SpellActivationOverlayFrame:SetPoint("center", UIParent)
   SpellActivationOverlayFrame:SetSize(256, 256)
   SpellActivationOverlayFrame:SetFrameStrata('MEDIUM')
   -- return overlay frame to original handler
   SpellActivationOverlayFrame:SetScript("OnEvent", SpellActivationOverlay_OnEvent)
end

function KokoSpellAuras:PLAYER_REGEN_DISABLED(event, ...)
   KokoSpellAuras:ToggleOverlayFrame()
end

function KokoSpellAuras:PLAYER_REGEN_ENABLED(event, ...)
   -- process new auras in queue
   for index, newAura in pairs(NewAuraQueue) do
      local specID, spellID, tex, pos, scale, r, g, b, custom = unpack(newAura)
      -- register shared media, insert into texture location table
      ns.RegisterTexture(tex, select(1, GetSpellInfo(spellID)))

      ns.texPosInsert(ns.BlizTexName(tex, spellID),pos)
      -- register defaults
      ns.AuraDefaults(custom or false, pClass, specID, spellID, tex, pos, scale, r, g, b)
   end

   -- rebuild options menu due to new aura
   ns.buildMenu()
   -- release
   self:UnregisterEvent('PLAYER_REGEN_ENABLED')
   NewAuraQueue = {}
end
