local addon, ns = ...

-- got Class??
local pClass = select(2,UnitClass("player"))

-- default constants
ns.sizeScale = 0.8;
ns.defOverlaySize = 256

local longSide = 256
local shortSide = 128

-- Addon save variables
ns.DB = function(val)
   if (not KokoSpellAuras_DB) or val then
      KokoSpellAuras_DB = {
         ['enable'] = true,
         ['aCol'] = true,
         ['OverlaySize'] = {
            ['strata'] = 'MEDIUM',
            ['height'] = 256,
            ['width'] = 256,
         },
      }
      -- loads defaults for auras in database
      ns.DBResetAura() 
   end

   return KokoSpellAuras_DB
end

ns.DBResetAura = function()
   KokoSpellAuras_DB['Auras'] = {
      ['WARRIOR'] = {},
      ['PALADIN'] = {},
      ['HUNTER'] =  {},
      ['ROGUE'] =  {},
      ['PRIEST'] = {},
      ['DEATHKNIGHT'] = {},
      ['SHAMAN'] = {},
      ['MAGE'] = {},
      ['WARLOCK'] = {},
      ['MONK'] = {},
      ['DRUID'] = {},
      ['DEMONHUNTER'] = {},
   }
   -- loads defaults for auras in database
   for pClass, specList in pairs(ns.AuraDB()) do
      for specID, auraList in pairs(ns.AuraDB()[pClass]) do
         for auraID, settingsList in pairs(ns.AuraDB()[pClass][specID]) do
            ns.AuraDefaults(pClass, specID, auraID, settingsList[1])
         end
      end
   end 
   for pClass, specList in pairs(ns.CustomDB()) do
      for specID, auraList in pairs(ns.CustomDB()[pClass]) do
         for auraID, settingsList in pairs(ns.CustomDB()[pClass][specID]) do
            ns.AuraDefaults(pClass, specID, auraID, settingsList[1])
         end
      end
   end 
end

-- Spell Auras database
ns.AuraDB = function(val)
   if (not KokoSpellAurasCollection_DB) or val then
      KokoSpellAurasCollection_DB = {
         ['WARRIOR'] = {},
         ['PALADIN'] = {},
         ['HUNTER'] =  {},
         ['ROGUE'] =  {},
         ['PRIEST'] = {},
         ['DEATHKNIGHT'] = {},
         ['SHAMAN'] = {},
         ['MAGE'] = {},
         ['WARLOCK'] = {},
         ['MONK'] = {},
         ['DRUID'] = {},
         ['DEMONHUNTER'] = {},  
      }
   end   
   
   return KokoSpellAurasCollection_DB
end

ns.CustomDB = function(val)
   if (not KokoSpellAurasCustom_DB) or val then
      KokoSpellAurasCustom_DB = {
         ['WARRIOR'] = {},
         ['PALADIN'] = {},
         ['HUNTER'] =  {},
         ['ROGUE'] =  {},
         ['PRIEST'] = {},
         ['DEATHKNIGHT'] = {},
         ['SHAMAN'] = {},
         ['MAGE'] = {},
         ['WARLOCK'] = {},
         ['MONK'] = {},
         ['DRUID'] = {},
         ['DEMONHUNTER'] = {},  
      }
   end   
   
   return KokoSpellAurasCustom_DB
end

ns.CleanUp = function() 
   for pClass, specList in pairs(ns.AuraDB()) do
      for specID, auraList in pairs(ns.AuraDB()[pClass]) do
         for auraID, settingsList in pairs(ns.AuraDB()[pClass][specID]) do
			if not GetSpellInfo(auraID) then
				-- spell no longer exists in game, remove from collection/config
				auraList[auraID] = nil
				ns.DB().Auras[pClass][specID][auraID] = nil
			end
         end
      end
   end 
end

-- location tables
ns.locationTable = {
   'Top',
   'Bottom',
   'Left',
   'Right',
   'Center',
   'TopLeft',
   'TopRight',
   'BottomLeft',
   'BottomRight',
   'Top + Bottom (Flipped)',
   'Left + Right (Flipped)',
   'Right (Flipped)',
   'Bottom (Flipped)',
}

ns.locationTableShort = {
   ['Top'] = 'TOP',
   ['Bottom'] = 'TOP',
   ['Left'] = 'SIDES',
   ['Right'] = 'SIDES',
   ['Center'] = 'CENTER',
   ['TopLeft'] = 'CORNER',
   ['TopRight'] = 'CORNER',
   ['BottomLeft'] = 'CORNER',
   ['BottomRight'] = 'CORNER',
   ['Top + Bottom (Flipped)'] = 'TOP',
   ['Left + Right (Flipped)'] = 'SIDES',
   ['Right (Flipped)'] = 'SIDES',
   ['Bottom (Flipped)'] = 'TOP',
   ['Sides'] = 'SIDES',
}

ns.texPosTable = {
   ['TOP'] = {},
   ['SIDES'] = {},
   ['CENTER'] = {},
   ['CORNER'] = {},
}

ns.complexLocationTable = {
	["RIGHT (FLIPPED)"] = {
		RIGHT = { hFlip = true },
	},
	["BOTTOM (FLIPPED)"] = {
		BOTTOM = { vFlip = true },
	},
	["LEFT + RIGHT (FLIPPED)"] = {
		LEFT = {},
		RIGHT = { hFlip = true },
	},
	["TOP + BOTTOM (FLIPPED)"] = {
		TOP = {},
		BOTTOM = { vFlip = true },
	},
}

ns.strataTable = {
   ['PARENT'] = "PARENT",
   ['BACKGROUND'] = "BACKGROUND",
   ['LOW'] = "LOW",
   ['MEDIUM'] = "MEDIUM",
   ['HIGH'] = "HIGH",
   ['DIALOG'] = "DIALOG",
   ['FULLSCREEN'] = "FULLSCREEN",
   ['FULLSCREEN_DIALOG'] = "FULLSCREEN_DIALOG",
   ['TOOLTIP'] = "TOOLTIP",
}

ns.texSizeTable = {
   ['TOP'] = {
      HEIGHT = shortSide,
      WIDTH = longSide,
   },
   ['SIDES'] = {
      HEIGHT = longSide,
      WIDTH = shortSide,
   },
   ['CORNER'] = {
      HEIGHT = shortSide,
      WIDTH = shortSide,
   },
   ['CENTER'] = {
      HEIGHT = longSide,
      WIDTH = longSide,
   }
}
ns.texSizeTable['DEFAULT'] = ns.texSizeTable['TOP']

ns.getTexSize = function(name)
   local pos = 'DEFAULT'
   
   for index, val in pairs(ns.texPosTable) do
      
      if tContains(val, name) then
         pos = index
         break
      end
   end
   return ns.texSizeTable[pos].HEIGHT or 128, ns.texSizeTable[pos].WIDTH or 128
end

ns.getRandomTex = function(location)
   if ns.texPosTable[location] then 
      local index = math.random(1, #ns.texPosTable[location])
      return ns.texPosTable[location][index]
   else
      return false
   end
end

ns.texPosInsert = function(name, location)
   if not ns.locationTableShort[location] then return end
   if tContains(ns.texPosTable[ns.locationTableShort[location]], name) then return end
   
   table.insert(ns.texPosTable[ns.locationTableShort[location]], name)
end

-- Sets a specified Aura default values
ns.AuraDefaults = function(custom, pClass, specID, spellID, tex, pos, scale, r, g, b)
   if not (pClass or specID or spellID or tex) then return end
   
   local DB = ns.DB()['Auras'][pClass]
   local AuraDB, AuraPosDB
	if custom then
		AuraDB = ns.CustomDB()[pClass]
		AuraPosDB = ns.CustomDB()[pos]
	else
		AuraDB = ns.AuraDB()[pClass]
		AuraPosDB = ns.AuraDB()[pos]
	end
   local shortname = ns.BlizTexName(tex, spellID)
   
   if not DB[specID] then DB[specID] = {} end
   if not DB[specID][spellID] then DB[specID][spellID] = {} end
   
   if (AuraDB[specID] and AuraDB[specID][spellID]) then
      -- collected defaults exist
      DB[specID][spellID][1] = ns.BlizTexName(AuraDB[specID][spellID][1], spellID)
      for i = 2, #AuraDB[specID][spellID] do
         DB[specID][spellID][i] = AuraDB[specID][spellID][i]
      end
   else
      DB[specID][spellID] = {
         shortname,
         pos or 'Top',
         scale or 1,
         r or 255,
         g or 255,
         b or 255,
      }
   end
   --insert addon specifics
   DB[specID][spellID][7] = false   -- hflip
   DB[specID][spellID][8] = false   -- vflip
   DB[specID][spellID][9] = 'None'  -- sound
   -- height and width settings
   DB[specID][spellID][10] = ns.texSizeTable[ns.locationTableShort[DB[specID][spellID][2]]].HEIGHT
   DB[specID][spellID][11] = ns.texSizeTable[ns.locationTableShort[DB[specID][spellID][2]]].WIDTH
end