local addon, ns = ...

-- Shared Media
local media = LibStub("LibSharedMedia-3.0")

-- media functions
ns.BlizTexName = function(texture, spellID)
   if not texture then return end
   if tonumber(texture) then
      if tonumber(spellID) then
         local n = GetSpellInfo(spellID)
         return GetSpellInfo(spellID) or ('FileID: ' .. texture)
      else
         return spellID
      end
   else
      return string.sub(texture, 34,-5)
   end
end

ns.TextureListing = function()
   local hash = media:HashTable('spelloverlay')
   local list = {}
   
   for index, val in pairs(hash) do
      list[index] = index
   end
   
   return list
end

ns.SoundListing = function()
   local hash = media:HashTable('sound')
   local list = {}
   
   for index, val in pairs(hash) do
      list[index] = index
   end
   
   return list
end

ns.RegisterTexture = function(path, name)
   media:Register('spelloverlay', name or ns.BlizTexName(path), path)
end
-------------- Media

-------------- Sounds
local SOUND_NAME = {
		"Rubber Ducky",
		"Cartoon FX", 
		"Explosion", 
		"Shing!",
		"Wham!",
		"Simon Chime",
		"War Drums",
		"Cheer",
		"Humm",
		"Short Circuit",
		"Fel Portal",
		"Fel Nova",
		"Argh!",
		"Bzz Zap!",
      "None",
};
local SOUND_PATH = {
		[[Sound\Doodad\Goblin_Lottery_Open01.ogg]],
		[[Sound\Doodad\Goblin_Lottery_Open03.ogg]],
		[[Sound\Doodad\Hellfire_Raid_FX_Explosion05.ogg]],
		[[Sound\Doodad\PortcullisActive_Closed.ogg]],
		[[Sound\Doodad\PVP_Lordaeron_Door_Open.ogg]],
		[[Sound\Doodad\SimonGame_LargeBlueTree.ogg]],
		[[Sound\Event Sounds\Event_wardrum_ogre.ogg]],
		[[Sound\Event Sounds\OgreEventCheerUnique.ogg]],
		[[Sound\Spells\SimonGame_Visual_GameStart.ogg]],
		[[Sound\Spells\SimonGame_Visual_BadPress.ogg]],
		[[Sound\Spells\Sunwell_Fel_PortalStand.ogg]],
		[[Sound\Spells\SeepingGaseous_Fel_Nova.ogg]],
		[[Sound\Events\EbonHold_ManScream3_08.ogg]],
		[[Sound\Spells\Ulduar_IronConcil_OverloadEffect.ogg]],
      [[]],
};

-------------- Overlays
local BLIZ_OVERLAYS = {
-- Generic Top
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericTop_01.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericTop_02.BLP",
-- Generic Arc
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_01.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_02.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_03.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_04.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_05.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\GenericArc_06.BLP",
-- Hunter
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Master_Marksman.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Focus_Fire.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Lock_And_Load.BLP",
-- Warrior
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Blood_Surge.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Sword_And_Board.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Sudden_Death.BLP",
-- Shaman
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Maelstrom_Weapon.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Maelstrom_Weapon_1.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Maelstrom_Weapon_2.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Maelstrom_Weapon_3.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Maelstrom_Weapon_4.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Fulmination.BLP",
-- Mage
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Impact.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Frozen_Fingers.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Arcane_Missiles.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Hot_Streak.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Brain_Freeze.BLP",
-- Priest
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Surge_Of_Light.BLP",
-- warlock
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Molten_Core.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Backlash.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Imp_Empowerment.BLP",      -- currently used for Shaman Lava Surge proc.
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Nightfall.BLP",
-- druid
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Natures_Grace.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Berserk.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Tooth_And_Claw.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Predatory_Swiftness.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Feral_OmenOfClarity.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Eclipse_Sun.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Fury_Of_Stormrage.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Eclipse_Moon.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Shooting_Stars.BLP",
-- Death Knight
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Dark_Transformation.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Sudden_Doom.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Rime.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Killing_Machine.BLP",
-- Paladin
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Hand_Of_Light.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Daybreak.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Art_Of_War.BLP",
"TEXTURES\\SPELLACTIVATIONOVERLAYS\\Grand_Crusader.BLP",
}

-- load sounds
for i = 1, #SOUND_NAME do
   media:Register('sound', SOUND_NAME[i], SOUND_PATH[i])
end

-- load overlays
for i = 1, #BLIZ_OVERLAYS do
   ns.RegisterTexture(BLIZ_OVERLAYS[i])
end

-- Overlay texture position and sizes
ns.texPosInsert('GenericTop_01', 'Top')
ns.texPosInsert('GenericTop_02', 'Top')
ns.texPosInsert('GenericArc_01', 'Sides')
ns.texPosInsert('GenericArc_02', 'Sides')
ns.texPosInsert('GenericArc_03', 'Sides')
ns.texPosInsert('GenericArc_04', 'Sides')
ns.texPosInsert('GenericArc_05', 'Sides')
ns.texPosInsert('GenericArc_06', 'Sides')

ns.texPosInsert('Master_Marksman', 'Top')
ns.texPosInsert('Focus_Fire', 'Top')
ns.texPosInsert('Lock_And_Load', 'Top')

ns.texPosInsert('Blood_Surge', 'Top')
ns.texPosInsert('Sword_And_Board', 'Sides')
ns.texPosInsert('Sudden_Death', 'Sides')

ns.texPosInsert('Maelstrom_Weapon', 'Top')
ns.texPosInsert('Maelstrom_Weapon_1', 'Top')
ns.texPosInsert('Maelstrom_Weapon_2', 'Top')
ns.texPosInsert('Maelstrom_Weapon_3', 'Top')
ns.texPosInsert('Maelstrom_Weapon_4', 'Top')
ns.texPosInsert('Fulmination', 'Top')

ns.texPosInsert('Impact', 'Top')
ns.texPosInsert('Frozen_Fingers', 'Sides')
ns.texPosInsert('Arcane_Missiles', 'Sides')
ns.texPosInsert('Hot_Streak', 'Sides')
ns.texPosInsert('Brain_Freeze', 'Top')

ns.texPosInsert('Surge_Of_Light', 'Sides')

ns.texPosInsert('Molten_Core', 'Sides')
ns.texPosInsert('Backlash', 'Top')
ns.texPosInsert('Imp_Empowerment', 'Sides')
ns.texPosInsert('Nightfall', 'Sides')

ns.texPosInsert('Natures_Grace', 'Sides')
ns.texPosInsert('Predatory_Swiftness', 'Top')
ns.texPosInsert('Berserk.BLP', 'Top')
ns.texPosInsert('Tooth_And_Claw.BLP', 'Sides')
ns.texPosInsert('Feral_OmenOfClarity', 'Sides')
ns.texPosInsert('Eclipse_Sun', 'Sides')
ns.texPosInsert('Fury_Of_Stormrage', 'Top')
ns.texPosInsert('Eclipse_Moon', 'Sides')
ns.texPosInsert('Shooting_Stars', 'Top')

ns.texPosInsert('Dark_Transformation', 'Top')
ns.texPosInsert('Sudden_Doom', 'Sides')
ns.texPosInsert('Killing_Machine', 'Sides')
ns.texPosInsert('Rime', 'Top')

ns.texPosInsert('Hand_Of_Light', 'Top')
ns.texPosInsert('Daybreak', 'Sides')
ns.texPosInsert('Art_Of_War', 'Sides')
ns.texPosInsert('Grand_Crusader', 'Sides')