local addon, ns = ...

local media = LibStub("LibSharedMedia-3.0")

-- got Class??
local pClass = select(2,UnitClass("player"))

-- get set functions
-- Configuration settings index values
-- 1: texture
-- 2: position
-- 3: scale
-- 4: rgb
-- 5: hflip
-- 6: vflip
-- 7: sound
-- 8: height
-- 9: width
ns.configOpt = {'texture', 'position', 'scale', 'rgb', 'hflip', 'vflip', 'sound', 'height', 'width', 'reset', 'frame x', 'frame y'}
local resetcfm = false
ns.showoverlay = false

ns.GetOptions = {
   ['texture'] = function(pClass, specID, spellID) return ns.DB()['Auras'][pClass][specID][spellID][1] end,
   ['position'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][2] end,
   ['scale'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][3] end,
   ['rgb'] = function(pClass, specID, spellID) return ns.DB()['Auras'][pClass][specID][spellID][4]/255, ns.DB()['Auras'][pClass][specID][spellID][5]/255, ns.DB()['Auras'][pClass][specID][spellID][6]/255 end,
   ['hflip'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][7] end,
   ['vflip'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][8] end,
   ['sound'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][9] end,
   ['height'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][10] end,
   ['width'] = function(pClass, specID, spellID) return  ns.DB()['Auras'][pClass][specID][spellID][11] end,
   ['color'] = function(pClass, specID, spellID) return 1,1,1 end,
}

ns.SetOptions = {
   ['texture'] = function(pClass, specID, spellID, val) 
      ns.DB()['Auras'][pClass][specID][spellID][1] = val
      -- set texture height/width defaults
      local h, w = ns.getTexSize(val)
      ns.SetOptions['height'](pClass, specID, spellID, h)
      ns.SetOptions['width'](pClass, specID, spellID, w)    
    end,
   ['position'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][2] = val end,
   ['scale'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][3] = val end,
   ['rgb'] = function(pClass, specID, spellID, r, g, b)
      ns.DB()['Auras'][pClass][specID][spellID][4] = floor(r*255 + 0.5)
      ns.DB()['Auras'][pClass][specID][spellID][5] = floor(g*255 + 0.5)
      ns.DB()['Auras'][pClass][specID][spellID][6] = floor(b*255 + 0.5)
   end,
   ['hflip'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][7] = val end,
   ['vflip'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][8] = val end,
   ['sound'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][9] = val; PlaySoundFile(media:Fetch("sound", val)) end,
   ['height'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][10] = val end,
   ['width'] = function(pClass, specID, spellID, val) ns.DB()['Auras'][pClass][specID][spellID][11] = val end,
   ['color'] = function(pClass, specID, spellID, r,g,b) end,
}

ns.GetOverlayCfg = function(pClass,specID,spellID)
   local cfg = {}
   for i = 1, 6 do
      if i == 4 then
         local r,g,b = ns.GetOptions[ns.configOpt[i]](pClass,specID,spellID)
         table.insert(cfg,  floor(r*255 + 0.5))
         table.insert(cfg,  floor(g*255 + 0.5))
         table.insert(cfg,  floor(b*255 + 0.5))
      else
         table.insert(cfg, ns.GetOptions[ns.configOpt[i]](pClass,specID,spellID))
      end
   end
   table.insert(cfg, ns.GetOptions['height'](pClass,specID,spellID))
   table.insert(cfg, ns.GetOptions['width'](pClass,specID,spellID))
   
   return unpack(cfg)
end

ns.options = { 
   name = "KokoSpellAuras",
   handler = KokoSpellAuras,
   type = "group",
   args = {}
}

ns.options.args = {
   config = {
      name = "Global Options",
      type = 'group',
      childGroups ='tab',
      order = 1,
      args = {
         gen = {
            name = 'General',
            type = 'group',
            order = 1,
            args = {
               enable = {
                  name = 'Enable',
                  desc = 'Enable KokoSpellAuras',
                  type = 'toggle',
                  order = 1,
                  get = function(info) 
                     return ns.DB().enable
                  end,
                  set = function(info, val)
                     if val then KokoSpellAuras:OnEnable() else KokoSpellAuras:OnDisable() end
                     ns.DB().enable = val
                  end,
               },
               reset = {
                  name = "Defaults",
                  desc = 'Reset settings to defaults.\nWarning - This also resets aura configuration.',
                  type = 'execute',
                  order = 3,
                  func = function(info, val)
                     ns.DB(true)
                     ns.buildMenu()
                  end,
               },               
            },
         },
         overlay = {
            name = 'Overlay Frame',
            type = 'group',
            order = 2,
            args = {
               enable = {
                  name = 'Show Overlay',
                  desc = 'Shows overlay frame for resizing',
                  type = 'toggle',
                  order = 1,
                  get = function(info) 
                     return ns.showoverlay
                  end,
                  set = function(info, val)
                     KokoSpellAuras:ToggleOverlayFrame(val)
                  end,
               },
               strata = {
                  name = 'Strata',
                  desc = 'Resets Overlay Strata',
                  type = 'select',
                  order = 2,
                  values = function() return ns.strataTable end,
                  get = function(info) return ns.DB()['OverlaySize']['strata'] end,
                  set = function(info,val) 
                     ns.DB()['OverlaySize']['strata'] = val
                  end,
               },
               reset = {
                  name = 'Reset',
                  desc = 'Resets Overlay Size',
                  type = 'execute',
                  order = 4,
                  func = function()
                     ns.DB()['OverlaySize']['strata'] = 'MEDIUM'
                     ns.DB()['OverlaySize']['height'] = 256
                     ns.DB()['OverlaySize']['width'] = 256
                     KokoSpellAuras:OnDisable()
                  end,
               },               
            },
         },
         data = {
            name = 'Collected Data',
            type = 'group',
            order = 3,
            args = {
               aCol = {
                  name = "Collect Auras",
                  desc = 'Collect Spell Auras',
                  type = 'toggle',
                  order = 2,
                  width = 'full',
                  get = function(info) 
                     return ns.DB().aCol
                  end,
                  set = function(info, val)
                     ns.DB().aCol = val
                  end,
               },  
               cfmRest = {
                  name = 'Delete Data',
                  desc = 'Enables deletion of all collected data.\nWarning - This also resets aura configuration.',
                  type = 'toggle',
                  order = 4,
                  get = function(info) return resetcfm end,
                  set = function(info, val) resetcfm = val end,
               },
               resetAura = {
                  name = "Delete Data",
                  desc = 'Deletes all collected data',
                  type = 'execute',
                  order = 5,
                  hidden = function() return not resetcfm end,
                  func = function(info, val)
                     ns.AuraDB(true)
					 ns.CustomDB(true)
                     ns.DBResetAura()
                     ns.buildMenu()
                  end,
               },
            },
         },
      },
   },
}

-- options menu
ns.buildAuraOpt = function(pClass, specID, auraID, custom)
   local DB = ns.DB().Auras
   
   local tbl = {}
   
   tbl.image = {
      name = '',
      type = 'description',
      order = 1,
      image = function()
         local tex = media:Fetch('spelloverlay',ns.GetOptions['texture'](pClass, specID, auraID))
         local h, w = ns.getTexSize(ns.BlizTexName(tex))
         local sizescale = 0.6
         local ratio = w/h

        -- tostring(tex) allows use of FileID's by blizzard.
        return tostring(tex), w*sizescale, h*sizescale
      end,
   }
   tbl.overlay = {
      name = 'Overlay',
      type = 'group',
      order = 10,
      desc = 'Overlay Options',
      args = {},
   }  
   tbl.size = {
      name = 'Size & Color',
      type = 'group',
      order = 20,
      desc = 'Sizing Options',
      args = {},
   }

   local category = {
      ['overlay'] = {
         1,    -- texture
         2,    -- position
         5,    -- hflip
         6,    -- vflip
      },
      ['size'] = {
         3,    -- scale
         4,    -- rgb
         8,    -- height
         9,    -- width
      },
   }
      
   local tbl_names = {
      'texture',
      'location',
      'scale',
      'color',
      'H-Flip',
      'V-Flip',
      'sounds',
      'height',
      'width',
      'Reset',
   }
   local tbl_desc = {
      'Overlay Texture Selection.\n\nChanging a texture will change height/width values.',
      'Overlay Location',
      'Overlay Scale',
      'Overlay Color',
      'Horizontal Flip',
      'Vertical Flip',
      'Sound Selection',
      'Texture Height',
      'Texture Width',
      'Reset Overlay Settings',
   }
   local tbl_values = {
      { type = 'select', values = function() return ns.TextureListing() end},
      { type = 'select', values = function() local tmp = {}; for i, j in pairs(ns.locationTable) do tmp[j] = j end return tmp end},
      { type = 'range', min = 0.5, max = 3, step = 0.1 },
      { type = 'color' },
      { type = 'toggle' },
      { type = 'toggle' },
      { type = 'select', values = function() return ns.SoundListing() end},
      { type = 'range', min = 100, max = 350, step = 0.1 },
      { type = 'range', min = 100, max = 350, step = 0.1 },
      { type = 'execute', func = function(info, val) 
			if custom then
				ns.AuraDefaults(custom, pClass, specID, auraID, ns.CustomDB()[pClass][specID][auraID][1])
			else
				ns.AuraDefaults(false, pClass, specID, auraID, ns.AuraDB()[pClass][specID][auraID][1])
			end
		end },
    }
   
   for i = 1, #tbl_names do
      local pos = nil
      
      for index, cat_table in pairs(category) do
         if tContains(cat_table, i) then
            pos = index
            break
         end
      end
      local temp = {
         name = tbl_names[i]:gsub("^%l", string.upper),
         desc = tbl_desc[i]:gsub("^%l", string.upper),
         order = i*5,
      }
      
      if tbl_names[i] == 'reset' then
         temp.func = function() end
      else
         temp.set = function(info, ...) ns.SetOptions[ns.configOpt[i]](pClass, specID, auraID, ...) end
         temp.get = function(info) return ns.GetOptions[ns.configOpt[i]](pClass, specID, auraID) end
      end
      
      for m, n in pairs(tbl_values[i]) do
         temp[m] = n
      end

      if pos then
         tbl[pos].args[tbl_names[i]] = temp
      else
         tbl[tbl_names[i]] = temp
      end
   end
   
   return tbl
end

ns.buildMenu = function()
   -- got Class??
   local pClass = select(2,UnitClass("player"))
   
   local menu = {}
   for i,_ in pairs(ns.options.args) do
      if i ~= 'config' then ns.options.args[i] = nil end
   end
   
   for specID, aura_tbl in pairs(ns.AuraDB()[pClass]) do
      local _,specName = GetSpecializationInfoByID(specID)
      
      -- class headers
      ns.options.args[specName] = {
         name = specName,
         type = 'group',
         order = specID,
         childGroups = 'tab',
         hidden = function()
            if not GetSpecialization() then return true end
            if specID == GetSpecializationInfo(GetSpecialization()) then
               return false
            else
               return true
            end
         end,
         args = {}
      }
      
      -- auras menu
      for auraID, defaults in pairs(aura_tbl) do
         local spellname = GetSpellInfo(auraID)
         if spellname then
            ns.options.args[specName].args[tostring(auraID)] = {
               name = spellname,
               type = 'group',
               order = auraID,
               childGroups ='tab',
               args = ns.buildAuraOpt(pClass, specID, auraID, false),
            }
         end
      end
   end
   
    for specID, aura_tbl in pairs(ns.CustomDB()[pClass]) do
      local _,specName = GetSpecializationInfoByID(specID)
      -- class headers
      ns.options.args[specName..'Custom'] = {
         name = specName .. ' Custom',
         type = 'group',
         order = specID,
         childGroups = 'tab',
         hidden = function()
            if not GetSpecialization() then return true end
            if specID == GetSpecializationInfo(GetSpecialization()) then
               return false
            else
               return true
            end
         end,
         args = {}
      }
      
      -- auras menu
      for auraID, defaults in pairs(aura_tbl) do
         local spellname = GetSpellInfo(auraID)
         if spellname then
            ns.options.args[specName..'Custom'].args[tostring(auraID)] = {
               name = spellname,
               type = 'group',
               order = auraID,
               childGroups ='tab',
               args = ns.buildAuraOpt(pClass, specID, auraID, true),
            }
         end
      end
   end

   -- update
   LibStub("AceConfigRegistry-3.0"):NotifyChange("KokoSpellAuras")
end
